# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - ApproveChanges

    Permit the approval of an edited page queued by the page editor when an
    unprivileged user attempted to save a page. Since queued changes are placed
    in a subpage, this action just moves the queued page content into the
    existing page when approving the changes.

    @copyright: 2011, 2013 Paul Boddie <paul@boddie.org.uk>
    @license: GNU GPL (v2 or later), see COPYING.txt for details.
"""

Dependencies = ['pages']

from MoinMoin.action import ActionBase
from MoinMoin.Page import Page
from MoinMoin.PageEditor import PageEditor, conflict_markers
from MoinMoin.util import diff3
from MoinSupport import getPagesForSearch
from ApproveChangesSupport import *

# Action class and supporting functions.

class ApproveChanges(ActionBase):

    "An action which approves a queued page version."

    queued_content_directives = ["acl", "parent-revision", "unapproved-user", "unapproved-user-queue"]

    def __init__(self, pagename, request):
        ActionBase.__init__(self, pagename, request)
        _ = self._
        self.form_trigger = "execute"
        self.form_trigger_label = _("Execute the indicated changes")

    def get_revision(self):
        request = self.request
        form = get_form(request)

        # Get the revision or None.

        rev = form.get("rev")
        if rev is None:
            return self.page.current_rev()
        else:
            return int(rev[0])

    def get_body(self, rev):
        request = self.request
        page = Page(request, self.page.page_name, rev=rev)
        return page.get_raw_body()

    def get_form_html(self, buttons_html):
        _ = self._
        request = self.request
        fmt = request.formatter
        form = get_form(request)

        if not is_queued_changes_page(request, self.pagename):
            return fmt.paragraph(1) + fmt.text(_("This page does not show queued changes.")) + fmt.paragraph(0)

        rev = self.get_revision()

        # Get information about the queued changes.

        body = self.get_body(rev)
        _body, directives = remove_directives(body, self.queued_content_directives)

        # Get the target page's parent revision for the queued changes.

        target_page_name = get_target_page_name(self.page)
        target_page = PageEditor(request, target_page_name)

        current_rev = target_page.current_rev()
        parent_rev = int(directives.get("parent-revision", current_rev))

        # Get the user who submitted the changes.

        username = directives.get("unapproved-user")

        # Get other edit details, if requested.

        if form.get("show-edits") and username:
            queued_edits = getPagesForSearch("title:regex:%s/ApprovalQueue$" % username, request)
        else:
            queued_edits = []

        d = {
            "approval_label"        : escape(_("Make %s an approved user") % username),
            "approve_edit_label"    : escape(_("Approve the displayed page version")),
            "block_label"           : escape(_("Block user %s by disabling their account") % username),
            "buttons_html"          : buttons_html,
            "discard_edit_label"    : escape(_("Discard the displayed page version")),
            "editaction_label"      : escape(_("Edit action")),
            "nothing_label"         : escape(_("Take no action for now")),
            "rev"                   : escattr(rev),
            "notice"                : escape(_("The affected page has been edited since the queued changes were made.")),
            "show_edits_label"      : escattr(_("Show all queued edits by this user.")),
            "showing_edits_label"   : escape(_("Queued edits by this user")),
            "useraction_label"      : escape(_("User action")),
            }

        # Make sure that the radio buttons are selected.

        for value in "nothing", "approve", "block":
            d["useraction_%s" % value] = form.get("useraction", ["nothing"])[0] == value and "checked='checked'" or ""

        for value in "approve", "discard":
            d["editaction_%s" % value] = form.get("editaction", ["approve"])[0] == value and "checked='checked'" or ""

        # Prepare the output HTML.

        html = '''
<table>'''

        if parent_rev != current_rev:
            html += '''
    <tr>
        <td colspan="2"><strong>%(notice)s</strong></td>
    </tr>''' % d

        # Actions to be taken with the edit.

        html += '''
    <tr>
        <td class="label" rowspan="2"><label>%(editaction_label)s</label></td>
        <td><input name="editaction" type="radio" value="approve" %(editaction_approve)s /> <label>%(approve_edit_label)s</label></td>
    </tr>
    <tr>
        <td><input name="editaction" type="radio" value="discard" %(editaction_discard)s /> <label>%(discard_edit_label)s</label></td>
    </tr>''' % d

        # User information and actions.

        if username and not user_is_approved(request, username):
            if not queued_edits:
                html += '''
    <tr>
        <td></td>
        <td><input name="show-edits" type="submit" value="%(show_edits_label)s" /></td>
    </tr>''' % d
            else:
                html += '''
    <tr>
        <td class="label"><label>%(showing_edits_label)s</label></td>
        <td><ul>''' % d

                for queued_page in queued_edits:
                    html += '''
        <li>%s</li>''' % escape(queued_page.page_name)

                html += '''
        </ul></td>
    </tr>'''

            # Actions to be taken with the user.

            html += '''
    <tr>
        <td class="label" rowspan="3"><label>%(useraction_label)s</label></td>
        <td><input name="useraction" type="radio" value="nothing" %(useraction_nothing)s /> <label>%(nothing_label)s</label></td>
    </tr>
    <tr>
        <td><input name="useraction" type="radio" value="approve" %(useraction_approve)s /> <label>%(approval_label)s</label></td>
    </tr>
    <tr>
        <td><input name="useraction" type="radio" value="block" %(useraction_block)s /> <label>%(block_label)s</label></td>
    </tr>''' % d

        html += '''
    <tr>
        <td></td>
        <td class="buttons">
            %(buttons_html)s
        </td>
    </tr>
</table>''' % d

        if rev:
            html += '''
<input name="rev" type="hidden" value="%(rev)s" />''' % d

        return html

    def do_action(self):

        "Approve the page and move it into place."

        _ = self._
        request = self.request
        form = get_form(request)

        # Make sure that only suitably privileged users can perform this action.

        queued_changes_page = get_queued_changes_page(request)

        if not is_reviewer(request):
            return 0, _("Only page reviewers can perform this action.")

        # Edit the target page, using this page's content.
        # The current page must be a queued page version.

        if not is_queued_changes_page(request, self.pagename):
            return 0, _("This page is not queued for approval.")

        # First, the displayed revision of the queued changes page must be
        # retrieved.

        rev = self.get_revision()
        body = self.get_body(rev)

        # Remove any introduced directives.

        body, directives = remove_directives(body, self.queued_content_directives)

        # Get the target page's parent revision for the queued changes.

        target_page_name = get_target_page_name(self.page)
        target_page = PageEditor(request, target_page_name)

        current_rev = target_page.current_rev()
        parent_rev = int(directives.get("parent-revision", current_rev))

        # Get the user who submitted the changes.

        username = directives.get("unapproved-user")

        # Approve the user if requested, regardless of what happens below.

        if username and form.get("useraction", [""])[0] == "approve":
            add_to_group_page(request, username, get_approved_editors_group(request))

        # NOTE: Add user blocking.

        # Discard the edit if requested.

        if form.get("editaction", [""])[0] == "discard":

            # NOTE: The page could be deleted completely or certain revisions
            # NOTE: purged.
            # NOTE: (to-do/proper-queued-page-deletion.txt)

            current_page = PageEditor(request, self.pagename)
            current_page.deletePage(_("Changes to page discarded."))

            # Redirect to the target page.

            request.http_redirect(target_page.url(request))
            return 1, None

        # Where the parent revision differs from the current revision of the
        # page, attempt to merge the changes.

        conflict = False

        if parent_rev != current_rev:

            # The body of the parent revision of the target page, along with the
            # body of the current revision must be acquired.

            parent_body = Page(request, target_page_name, rev=parent_rev).get_raw_body()
            current_body = target_page.get_raw_body()

            # The parent, current and queued texts must then be merged.

            body = diff3.text_merge(parent_body, current_body, body, True, *conflict_markers)

            # Look for conflict markers and redirect to edit mode on the
            # resulting page if they are present.

            for marker in conflict_markers:
                if body.find(marker) != -1:
                    conflict = True
                    break

        # Delete the queued changes page.
        # NOTE: The page could be deleted completely or certain revisions
        # NOTE: purged.
        # NOTE: (to-do/proper-queued-page-deletion.txt)

        current_page = PageEditor(request, self.pagename)
        current_page.deletePage(_("Changes to page approved."))

        # Prepare a comment.

        comment = username and \
            _("Changes to page by %s approved from queue revision %d.") % (username, rev) or \
            _("Changes to page approved from queue revision %d.") % rev

        # Save the target page, but only if there is no conflict.

        if not conflict:

            # Switch user if a specific user was recorded.

            if username:
                new_user = get_user(request, username)
            else:
                new_user = None

            if new_user:
                user = request.user
                request.user = new_user

            # Save the page.

            try:
                try:
                    target_page.saveText(body, 0, comment=comment)
                except PageEditor.Unchanged:
                    pass

            # Restore the user.

            finally:
                if new_user:
                    request.user = user

            # Redirect to the target page.

            request.http_redirect(target_page.url(request))

        # Otherwise, send the page editor.
        # NOTE: Replacing the revision in the request to prevent Moin from
        # NOTE: attempting to use the queued changes page's revision.
        # NOTE: Replacing the action and page in the request to avoid issues
        # NOTE: with editing tickets.

        else:
            request.rev = current_rev
            request.action = "edit"
            request.page = target_page
            target_page.sendEditor(preview=body, comment=comment, staytop=True)

        return 1, None

    def render_success(self, msg, msgtype):

        """
        Render neither 'msg' nor 'msgtype' since redirection should occur
        instead.
        NOTE: msgtype is optional because MoinMoin 1.5.x does not support it.
        """

        pass

# Action function.

def execute(pagename, request):
    ApproveChanges(pagename, request).render()

# vim: tabstop=4 expandtab shiftwidth=4
